;*****************************************************************************
;*                            TEMPERATUR-REGLER FUER PFEIFER                 *
;*                              02.01.14                                     *
;*****************************************************************************

        LIST    P=12F675       
        TITLE   "TEMPERATUR-REGLER"

        ERRORLEVEL -302
        INCLUDE <P12F675.INC>

__CONFIG _INTRC_OSC_NOCLKOUT & _WDT_OFF & _PWRTE_OFF & _BODEN_OFF & _MCLRE_OFF & _CPD_OFF & _CP_OFF





;------------------------- MACROS -----------------------------------------

BANK0                   MACRO                   ;MACRO TO SELECT DATA RAM BANK 0
                        BCF     STATUS,RP0
                        ENDM
;----------------------------------------------------------------------------
BANK1                   MACRO                   ;MACRO TO SELECT DATA RAM BANK 1
                        BSF     STATUS,RP0
                        ENDM
;----------------------------------------------------------------------------


;--------------------------------------------------------------------------
;REGISTER ZUORDNEN NACH DEM DIE FESTEN-REGISTER ZU ENDE SIND
;ES GEHT ALSO ERST AB 22 HEX LOS
;DIES IST ALSO DIE RAM-SPEICHER ZUORDNUNG

MERK                    EQU     0X22             
ZAE                     EQU     0X23
AD_H                    EQU     0X24
AD_L                    EQU     0X25



TEMPERATUR_WERT         EQU     D'56'

;JETZT TEMPERATURWERT FR 150C, ALSO DIE D'56' EINGESTELLT
;
;BERECHNUNG:
;
;PT100 WIDERSTAND BERECHNEN UNTER 
;HTTP://WWW.PT100.DE/WIDERSTAND.HTM
;WIDERSTAND BERECHNEN FR C
;BEI 150C IST ER 157 OHM VOM PT100

;SPANNUNGS-EINGANG AM   AN0-EINGANG   MIT 2,7 KOHM WIDERSTAND ALS SPANNUNGSTEILER 
;BERECHNET SICH :
;U=  [ 157 OHM : (157 OHM + 2,7 KOHM) ] * 5,0 VOLT
;U= 275 MVOLT
;BEI GRER 275 MVOLT AN AN0 SIND ES GRER ALS 150C, DANN SCHALTET ES

;BIT BERECHNUNG:
;BIT-ZAHL = (275 MVOLT * 1024) : 5,0VOLT = 56
;DIE 56 BEI TEMPERATURWERT OBEN EINTRAGEN ALS D'56'
;ES SCHALTET ALSO BEI GRSSER 150C 

;WEITERE TEMPERATUR-SCHALTWERTE, WELCHE MAN OBEN EINTRAGEN KNNTE
; 50C = 119 OHM = 211 MVOLT = D'43'
;100C = 138 OHM = 243 MVOLT = D'50'
;150C = 157 OHM = 275 MVOLT = D'56'
;200C = 176 OHM = 306 MVOLT = D'63'
;250C = 194 OHM = 335 MVOLT = D'69'
;300C = 212 OHM = 364 MVOLT = D'74'
;350C = 230 OHM = 392 MVOLT = D'80'
;400C = 247 OHM = 419 MVOLT = D'86'
;450C = 264 OHM = 445 MVOLT = D'91'

;MAN BEDENKE ALS BEISPIEL :
;ZWISCHEN 400C UND 450C SIND ES ZWAR 50C , ABER
;DER GEMESSENE BIT-WERT BEIM ANALOG-DIGITAL-WANDLER IST NUR VON 86 BIS 91 = 5
;ALSO NUR 5 ZAHLENWERTE (ZWISCHEN 400C BIS 450C)
;ES WIRD ALSO NUR ALLER 10C EIN UNTERSCHIEDLICHER WERT GEMESSEN.
;MAN KANN HIER NICHT AUF 1C GENAU MESSEN, SONDERN NUR MIT EINER GENAUIGKEIT VON GRSSER 10C
;--------------------------------------------------------------------------


;SPEICHERANFANG BZW. BEGINN
    ORG 0X0000
    GOTO MAIN           ; SPRUNG ZUM HAUPTPROGRAMM
    ORG 0X0004
    RETFIE              ; SOFORT INTERRUPT

    ORG 0X0010
;*****************************************************************************
;*                            HAUPTPROGRAMM                                  *
;*                              02.01.14                                     *
;*****************************************************************************
;PORT-INITIALISIERUNG
MAIN                    BANK0
                        CLRF    GPIO     
                        MOVLW   0X07                        ;COMPARATORS ALLE AUS 
                        MOVWF   CMCON

                        BANK1
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
                        MOVLW   B'00110001'                 ;ANALOG/DIGITAL
                        MOVWF   ANSEL                       ;GP2 + GP1 + GP4 + GP5 DIGITAL I/O     AN0 -ANALOG/DIGITAL WANDLER MIT RC-TAKT-INTERN  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                        MOVLW   B'11001001'                 ;GP2 + GP1 + GP4 + GP5 ALS AUSGANG (ALSO HIER DIE 0), SONST ALLES EINGNGE (ALSO IMMER DIE (1))
                        MOVWF   TRISIO

;KALIBRIERUNGSWERT FR DEN INTERNEN OSZILLATOR AM ENDE DES HAUPTSPEICHERS BERNEHMEN
                        CALL    0X03FF                  
                        MOVWF   OSCCAL

;INIT TIMER 0 FUER 20MSEC FUER ZEITSCHLEIFE VOREINGESTELLT 
;(HIER MUSS MAN IN DEM DATENBLATT (DATASHEET) BEI WWW.MICROCHIP.COM (EIN *.PDF-FILE) NACHSEHEN, WIE ALLES FUNKTIONIERT)
                        MOVLW   B'10000111'                 ;BIT0-2 111=PRESCALER256, BIT3 0=TMR0, BIT4 0=ENTFAELLT, BIT5 0=TAKT VOM SYSTEM, BIT6 0=KEIN RB-INTERRUPT, BIT7 1=PULL-UPS DISABLE 
                        MOVWF   OPTION_REG      
                        BANK0

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;ANALOG/DIGITAL
                        MOVLW   B'10000000' ;ADMF=1 RECHTSBNDIG, VDD ALS REF, CHANNEL AN0, NO GO BIT1, NO ADON BIT0
                        MOVWF   ADCON0

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                        CALL    AUS

;INITIALISIERUNG--ZU--ENDE
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;HAUPTPROGRAMM

ANFANG              MOVLW   D'50'           ;ZEITSCHLEIFE FUER 1SEC EINGESTELLT    50*20MSEC = 1000MSEC = 1SEC
                    CALL    ZEITSCHL        ;AN STELLE VON D'50' KNNEN ZAHLENWERTE VON D'1' BIS D'255' EINGETRAGEN WERDEN
                                            ;DIE ZEITSCHLEIFE HAT DANN DEM ENTSPRECHEND EINE ANDEREN ZEIT

                    CALL    AD_SPANNUNG     ;HIER WIRD DIE SPANNUNG AM PT100 GEMESSEN UND DANN GESCHALTET
                    GOTO    ANFANG


;*****************************************************************************
;*                              AD-MESSEN                                    *
;*                              26.08.13                                     *
;*****************************************************************************
AD_SPANNUNG     BCF     PIR1, ADIF

;ANALOG/DIGITAL-WANDLUNG STARTEN
                BSF     ADCON0, ADON        
                CALL    MUE40               ;ZEITSCHLEIFE MUSS SEIN
                BSF     ADCON0, GO_DONE     ;GEHT LOS

III             BTFSS  PIR1, ADIF           ;ES MISST
                GOTO   III

;ANALOG/DIGITAL-WANDLUNG STOPPEN
                BCF     ADCON0, ADON        
                BCF     ADCON0, GO_DONE    


;AD-ERGEBNIS ABSPEICHERN IN RAM-ZELLEN
                MOVF    ADRESH, W
                MOVWF   AD_H
                BANK1
                MOVF    ADRESL, W
                BANK0
                MOVWF   AD_L


;BERECHNUNG
;WENN AD-HIGH GLEICH 0X00 IST, DANN WEITER ZU AD-LOW-VERGLEICH
                MOVLW   0X00
                SUBWF   AD_H, W
                BTFSC   STATUS, Z
                GOTO    WEITER

                CALL    EIN              ;BEI GRSSER 0X00 (SEHR HOHE TEMPERATUR) / ES SCHALTET EIN
                RETURN

;WENN AD-LOW GRSSER IST, DANN EIN-SCHALTEN
WEITER          MOVLW   TEMPERATUR_WERT
                SUBWF   AD_L, W
                BTFSC   STATUS, C
                GOTO    WEITER1

                CALL    AUS             ;WENN AD-LOW KLEINER IST, DANN AUS-SCHALTEN
                RETURN

WEITER1         CALL    EIN             ;WENN AD-LOW GRSSER IST, DANN EIN-SCHALTEN
                RETURN

;*****************************************************************************
;*                              EIN                                          *
;*                              04.05.12                                     *
;*****************************************************************************
EIN
    BSF     GPIO, GP2
    BSF     GPIO, GP1
    BCF     GPIO, GP5
    BCF     GPIO, GP4
    RETURN

;*****************************************************************************
;*                              AUS                                          *
;*                              04.05.12                                     *
;*****************************************************************************
AUS
    BCF     GPIO, GP2
    BCF     GPIO, GP1
    BSF     GPIO, GP5
    BSF     GPIO, GP4
    RETURN

;*****************************************************************************
;*                              40SEC ZEITSCHLEIFE                          *
;*                              31.10.03                                     *
;*****************************************************************************

MUE40       MOVLW   D'4'
            MOVWF   ZAE
MUE         NOP             ;10X NOP ALSO T = 10 * 0.25SEC = 10S  * 4 = 40SEC  
            NOP         
            NOP
            NOP
            NOP
            NOP
            NOP
            NOP
            NOP
            NOP
            DECFSZ  ZAE, F
            GOTO    MUE
            RETURN

;*****************************************************************************
;*                              20MSEC ZEITSCHLEIFE                          *
;*                              05.05.12                                     *
;*****************************************************************************
ZEITSCHL                MOVWF   MERK

T01                     MOVLW   D'178'
                        MOVWF   TMR0                    ;DER FESTE RAM-SPEICHER (TMR0) WIRD MIT 217 GELADEN
                        BCF     INTCON, T0IF            ;OVERFLOW = 0
                
T02                     BTFSS   INTCON, T0IF            ;WENN DIE ZEIT VON 10MSEC ABGELAUFEN IST, WIRD T0IF HIGH (1) UND 
                        GOTO    T02                     ;ES ERFOLGT EIN SPRUNG ZU "DECFSC MERK, F"
 
                        DECFSZ  MERK, F                 ;ES WIRD DER WERT IN "MERK" ABWRTS GEZHLT, BIS ER (0) IST UND
                        GOTO    T01                     ;DANN ERFOLGT EIN SPRUNG ZU "RETURN"-ENDE DES UNTERPROGRAMMS

                        RETURN

;ZEITBERECHNUNG FUER 20MS
;(HIER MUSS MAN IN DEM DATENBLATT (DATASHEET) BEI WWW.MICROCHIP.COM (EIN *.PDF-FILE) NACHSEHEN, WIE ALLES FUNKTIONIERT)

;PRESCALER 256X  *  SYSTEM 4X  * TAKTZEIT ====> 256*4*0.25SEC=256SEC   /DIE TAKTZEIT ERGIBT SICH AUS DEM INTERNEN OSZILLATOR
;20MS : 256SEC= 78                                                      /TAKTZEIT = 1 : F  (F)-OSZILLATORFREQUENZ JETZT 4MHZ
;ALSO
;256S*D'78'=CA.20MS
;TMR0 ALSO D'256'-D'78'=D'178' ENTSPRICHT CA. 20MS
;178 DEZIMAL OBEN EINTRAGEN ALS D'178'



                        END

